﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright company="DHGMS Solutions" file="Date.cs">
//   2004-2012 DHGMS Solutions. Some Rights Reserved. Licensed under GNU General Public License version 2 (GPLv2)
// </copyright>
// --------------------------------------------------------------------------------------------------------------------

namespace Dhgms.TplHelper.Controller.Task.Sql
{
    using System.Data.Common;

    using Dhgms.DataManager.Model.Helper.Database;
    using Dhgms.TplHelper.Model.Info.TaskResult;

    /// <summary>
    /// Base class for an SQL Task that deals with a date
    /// </summary>
    /// <typeparam name="TDatabaseHelperClass">
    /// The DB Helper Class used for commands
    /// </typeparam>
    /// <typeparam name="TParameterClass">
    /// The .NET DB Parameter Type
    /// </typeparam>
    public abstract class Date<TDatabaseHelperClass, TParameterClass> :
        Base<SingleResultDateTime, TDatabaseHelperClass, TParameterClass>
        where TDatabaseHelperClass : Base<TParameterClass>, new() where TParameterClass : DbParameter
    {
        #region Constructors and Destructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Date{TDatabaseHelperClass,TParameterClass}"/> class.
        /// </summary>
        /// <param name="taskName">
        /// The task name.
        /// </param>
        /// <param name="connectionString">
        /// The connection string.
        /// </param>
        protected Date(string taskName, string connectionString)
            : base(taskName, connectionString)
        {
        }

        #endregion

        #region Methods

        /// <summary>
        /// The OnProcess event for doing the donkey work
        /// </summary>
        /// <param name="result">
        /// The result.
        /// </param>
        protected override void OnProcess(SingleResultDateTime result)
        {
            result.Result = this.DbHelper.GetDate(this.ConnectionString, this.GetSql(), this.GetParameters(), false);
        }

        #endregion
    }
}