﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright company="DHGMS Solutions" file="EmbeddedStonglyTypedCollection.cs">
//   2004-2012 DHGMS Solutions. Some Rights Reserved. Licensed under GNU General Public License version 2 (GPLv2)
// </copyright>
// --------------------------------------------------------------------------------------------------------------------

namespace Dhgms.TplHelper.Controller.Task.Sql
{
    using System.Data.Common;
    using System.Reflection;

    using Dhgms.DataManager.Model.Helper.Database;
    using Dhgms.TplHelper.Model.Info.TaskResult;

    /// <summary>
    /// Task for pulling an Embedded SQL query into a stongly typed collection
    /// </summary>
    /// <typeparam name="TDatabaseHelperClass">
    /// The type for the database helper
    /// </typeparam>
    /// <typeparam name="TInformationClass">
    /// The type for the information class
    /// </typeparam>
    /// <typeparam name="TParameterClass">
    /// The type for the SQL Parameters
    /// </typeparam>
    /// <typeparam name="TAdoNetHelperClass">
    /// ADO.NET helper class
    /// </typeparam>
    public abstract class EmbeddedStonglyTypedCollection<TDatabaseHelperClass, TInformationClass, TParameterClass, TAdoNetHelperClass> :
        Embedded<StronglyTypedCollection<TInformationClass>, TDatabaseHelperClass, TParameterClass>
        where TDatabaseHelperClass : Base<TParameterClass>, new()
        where TInformationClass : DataManager.Model.Info.Base<TInformationClass>, new()
        where TParameterClass : DbParameter
        where TAdoNetHelperClass : DataManager.Model.Helper.AdoNet.Base<TInformationClass>, new()
    {
        #region Constructors and Destructors

        /// <summary>
        /// Initializes a new instance of the <see cref="EmbeddedStonglyTypedCollection{TDatabaseHelperClass,TInformationClass,TParameterClass,TAdoNetHelper}"/> class. 
        /// </summary>
        /// <param name="taskName">
        /// Name of the task
        /// </param>
        /// <param name="connectionString">
        /// Database connection string
        /// </param>
        /// <param name="assembly">
        /// The assembly containing the resource
        /// </param>
        /// <param name="resourceNamespace">
        /// Namespace where the resource file resides. Usually the application name
        /// </param>
        /// <param name="resourceFileName">
        /// The filename of the embedded resource
        /// </param>
        /// <param name="assumeSqlSafe">
        /// Whether to skip the SQL injection safety check.  Use only if you know the code you are passing in is safe and contains a quoted string that is preventing it passing the saftey check
        /// </param>
        protected EmbeddedStonglyTypedCollection(
            string taskName, 
            string connectionString, 
            Assembly assembly, 
            string resourceNamespace, 
            string resourceFileName, 
            bool assumeSqlSafe)
            : base(taskName, connectionString, assembly, resourceNamespace, resourceFileName, assumeSqlSafe)
        {
        }

        #endregion

        #region Methods

        /// <summary>
        /// The OnProcess event for doing the donkey work
        /// </summary>
        /// <param name="result">
        /// object that contains result information
        /// </param>
        protected override void OnProcess(StronglyTypedCollection<TInformationClass> result)
        {
            result.Result = this.DbHelper.GetStronglyTypedCollection<TInformationClass, TAdoNetHelperClass>(
                this.ConnectionString, this.GetSql(), this.GetParameters(), this.AssumeSqlSafe);
        }

        #endregion
    }
}