﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="Task.cs" company="DHGMS Solutions">
//   2004-2012 DHGMS Solutions. Some Rights Reserved. Licensed under GNU General Public License version 2 (GPLv2)
// </copyright>
// <summary>
//   Helper methods for tasks
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Dhgms.TplHelper.Model.Helper
{
    using System;
    using System.Diagnostics;
    using System.Threading.Tasks;

    /// <summary>
    /// Helper methods for tasks
    /// </summary>
    public static class Task
    {
        /// <summary>
        /// Runs a task instantly
        /// </summary>
        /// <param name="task">
        /// The task to run.
        /// </param>
        /// <typeparam name="TTaskClass">
        /// The type of the task being run
        /// </typeparam>
        /// <typeparam name="TResultClass">
        /// The type of the result information being returned
        /// </typeparam>
        /// <returns>
        /// Job Result Information
        /// </returns>
        public static TResultClass Execute<TTaskClass, TResultClass>(TTaskClass task)
            where TResultClass : Info.TaskResult.Base, new()
            where TTaskClass : Controller.Task.Base<TResultClass>
        {
            if (task == null)
            {
                throw new ArgumentNullException("task");
            }

            if (Debugger.IsAttached)
            {
                return task.Execute();
            }

            var t = task.Task;
            t.RunSynchronously();

            if (!task.Task.IsFaulted)
            {
                return task.Task.Result;
            }

            if (t.Exception == null)
            {
                throw new TaskCanceledException();
            }

            if (t.Exception.InnerExceptions != null && t.Exception.InnerExceptions.Count == 1)
            {
                throw t.Exception.InnerExceptions[0];
            }

            throw t.Exception;

            /*
            var schedFact = new StdSchedulerFactory();
            var scheduler = schedFact.GetScheduler();
            scheduler.Start();

            var jobDetail = new JobDetailImpl(task.Name, "normal group", typeof(TTaskClass));

            var trigger = new SimpleTriggerImpl();

            var triggerFiredBundle = new TriggerFiredBundle(jobDetail, trigger, null, false, null, null, null, null);

            var jec = new JobExecutionContextImpl(
                scheduler,
                triggerFiredBundle,
                task);

            task.Execute(jec);

            return (TResultClass)jec.Result;
             * */
        }
    }
}
